package jeeafondo.webform;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import jeeafondo.webform.annotations.Form;
import jeeafondo.webform.annotations.OnEvent;
import jeeafondo.webform.annotations.Validate;


public abstract class WebForm
{
	private Hashtable<String,WebControl> controls;
	private Hashtable<WebControl,String> errors;
	private Hashtable<String,Screen> screens;
	
	private String name;
	private String page;
	
	protected WebForm()
	{
		controls = new Hashtable<>();
		errors = new Hashtable<>();
		screens = new Hashtable<>();
	}
	
	public void init(HttpSession session)
	{	
	}
	
	boolean validateControls()
	{
		boolean b = true;
		
		Enumeration<WebControl> e = controls.elements();
		while( e.hasMoreElements() )
		{
			WebControl w = e.nextElement();
		    if(  ValidableWebControl.class.isAssignableFrom(w.getClass()) )
			{
		    	if( w.isEnabled() )
		    	{
					ValidableWebControl vwc = (ValidableWebControl)w; 
					boolean aux = vwc.validate();
					if( !aux )
					{
						addError(vwc,vwc.getErrorMessage());
					}
					
					b = b && aux;
		    	}
			}
		}
		
		return b;
	}
	
	public boolean validate()
	{
		return true;
	}
	
	public static <T extends WebForm> T loadForm(Class<T> clazz, HttpServletRequest request)
	{
		try
		{
			String formName = clazz.getSimpleName();
			WebForm instance = (WebForm)request.getSession().getAttribute(formName);
			if( instance==null )
			{
				instance = clazz.newInstance();
				instance.name = formName;
				
				_addAnnotations(instance);
				_addControls(instance);
												
				// invoco al init
				instance.init(request.getSession());
				request.getSession().setAttribute(formName,instance);
			}
			
			return (T)instance;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			throw new RuntimeException(ex);
		}
	}	
	
	private static void _addAnnotations(WebForm instance)
	{
		Class<?> clazz = instance.getClass();

		// seteo el page de esta pagina
		Form form = null;
		for( Class<?> tmp=clazz; form==null && WebForm.class.isAssignableFrom(tmp); tmp = tmp.getSuperclass() )
		{	
			form = tmp.getAnnotation(Form.class);
		}
		
		// nombre de la pagina
		String page = form.page();
		instance.setPage(page);

		List<Field> fields = _obtenerFields(clazz);
		
		for(Field f:fields)
		{	
			// le asigno el nombre al control
			((WebControl)UBean.invokeGetter(instance,f.getName())).setName(instance.name+"."+f.getName());
			 
			// si es generador de eventos...
			//if( f.getClass().isAssignableFrom(EventWebComponent.class) )
			if( EventWebComponent.class.isAssignableFrom(f.getType()) )
			{			
				OnEvent onEvent = f.getAnnotation(OnEvent.class);				
				if( onEvent!=null )
				{
					EventWebComponent ewc = (EventWebComponent)UBean.invokeGetter(instance,f.getName());

					// seteo el metodo que debe invocarse al ocurrir el evento
					String mtdArg=onEvent.value().length()==0?"":"-"+onEvent.value();
					ewc.setPathOnEvent(onEvent.method()+mtdArg);
					
					// seteo si debe o no invocar las validaciones
					ewc.setEnableValidations(onEvent.validate());	
				}
			}
			
			// si es un control validable...
			if( ValidableWebControl.class.isAssignableFrom(f.getType()) )
			{
				Validate validate = f.getAnnotation(Validate.class);
				if( validate != null )
				{
					ValidableWebControl vwc = (ValidableWebControl)UBean.invokeGetter(instance,f.getName());
				
					// asigno la regexp y el error
					vwc.setRegexp(validate.regexp());
					vwc.setErrorMessage(validate.error());
				}
			}
		}
	}

	private static List<Field> _obtenerFields(Class<?> clazz)
	{
		Class<?> tmp = clazz;
		
		ArrayList<Field> fields = new ArrayList<>();
		while( !tmp.equals(WebForm.class) )
		{
			Field[] fs = tmp.getDeclaredFields();
			for(Field f:fs)
			{
				if( WebControl.class.isAssignableFrom(f.getType()) )
				{
					fields.add(f);
				}
			}
			
			tmp = tmp.getSuperclass();
		}
		
		
		return fields;
	}

	private static void _addControls(WebForm instance)
	{
		try
		{
			Method[] methods = instance.getClass().getMethods();
			for(Method m:methods)
			{
				if(WebControl.class.isAssignableFrom(m.getReturnType()) )
				{
					instance.addControl((WebControl)m.invoke(instance));
				}
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			throw new RuntimeException(ex);
		}
	}

	public void setPage(String p)
	{
		this.page=p;
	}
	
	public String getPage()
	{
		return this.page;
	}
	
	private void addControl(WebControl c)
	{
		c.setFormName(getName());
		controls.put(c.getName(),c);
	}
	
	@Override
	public String toString()
	{
		return name;
	}
	
	public String getName()
	{
		return name;
	}

	void setParamValues(HttpServletRequest request)
	{
		for(Enumeration<String> e = controls.keys(); e.hasMoreElements();)
		{
			String cName = e.nextElement();
			WebControl c = controls.get(cName);
			c.setValue(request.getParameter(c.getName()));
		}	
	}	
	
	public void addScreen(Screen s)
	{
		screens.put(s.getName(),s);
	}

	public void showScreen(String screenName)
	{
		Screen screen = screens.get(screenName);
		screen.applyStates();
	}
	
	public void addError(WebControl wc, String errorMssg)
	{
		errors.put(wc,errorMssg);
	}
	
	public String getError(WebControl wc)
	{
		String x = errors.get(wc);
		return x==null?"":x;
	}

	void clearErrors()
	{
		errors.clear();
	}
}
