package demo.webform;

import javax.servlet.http.HttpSession;

import jeeafondo.webform.Button;
import jeeafondo.webform.ListBox;
import jeeafondo.webform.Screen;
import jeeafondo.webform.TextField;
import jeeafondo.webform.WebForm;
import jeeafondo.webform.annotations.Form;
import jeeafondo.webform.annotations.OnEvent;
import jeeafondo.webform.annotations.Validate;

@Form(page="index.jsp")
public class MyForm extends WebForm
{
	@OnEvent(method="showScreen", value="ACTIVA")
	private Button bComenzar=new Button("Comenzar");

	@OnEvent(method="showScreen", value="INACTIVA")
	private Button bFinalizar=new Button("Finalizar");

	@OnEvent(method="agregaItemAction", validate=true)
	private Button bAgregar=new Button("Agregar");

	@OnEvent(method="borraItemsAction")
	private Button bBorrar=new Button("Eliminar Items");

	@OnEvent(method="seleccionaItemAction")
	private ListBox lstItems=new ListBox(5);

	@Validate(regexp=TextField.NOT_EMPTY, error="Debe ingresar un valor")
	private TextField tfItem=new TextField();

	@Override
	public void init(HttpSession session)
	{
		Screen s1=new Screen("INACTIVA");
		s1.addControl(bAgregar,false);
		s1.addControl(bBorrar,false);
		s1.addControl(tfItem,false,"");
		s1.addControl(lstItems,false);
		s1.addControl(bComenzar,true);
		s1.addControl(bFinalizar,false);
		addScreen(s1);

		Screen s2=new Screen("ACTIVA");
		s2.addControl(bAgregar,true);
		s2.addControl(bBorrar,true);
		s2.addControl(tfItem,true,"");
		s2.addControl(lstItems,true);
		s2.addControl(bComenzar,false);
		s2.addControl(bFinalizar,true);
		addScreen(s2);

		// defino que pantalla se mostrara al inicio
		showScreen("INACTIVA");
	}

	// bAgregar
	public void agregaItemAction()
	{
		// guardo el valor del textfield
		String item=tfItem.getValue();

		// lo asigno a la lista
		lstItems.addItem(item);

		// limpio el textfield
		tfItem.setValue("");
	}

	public void borraItemsAction()
	{
		// borro todos los items de la lista
		lstItems.removeItems();
	}

	// lstItems
	public void seleccionaItemAction()
	{
		// guardo el item seleccionado (su id)
		String aux=lstItems.getValue();

		// remuevo el item seleccionado
		lstItems.removeSelectedItem();

		// lo asigno como valor en el textfield
		tfItem.setValue(aux);
	}
	
	   @Override
	   public boolean validate()  
	   {
	      String item = tfItem.getValue();
	      if( lstItems.contains(item) )
	      {
	         addError(tfItem,"No se admiten items repetidos");
	         return false;
	      }

	      return true;
	   }

	public Button getBComenzar()
	{
		return bComenzar;
	}

	public void setBComenzar(Button bComenzar)
	{
		this.bComenzar=bComenzar;
	}

	public Button getBFinalizar()
	{
		return bFinalizar;
	}

	public void setBFinalizar(Button bFinalizar)
	{
		this.bFinalizar=bFinalizar;
	}

	public Button getBAgregar()
	{
		return bAgregar;
	}

	public void setBAgregar(Button bAgregar)
	{
		this.bAgregar=bAgregar;
	}

	public Button getBBorrar()
	{
		return bBorrar;
	}

	public void setBBorrar(Button bBorrar)
	{
		this.bBorrar=bBorrar;
	}

	public ListBox getLstItems()
	{
		return lstItems;
	}

	public void setLstItems(ListBox lstItems)
	{
		this.lstItems=lstItems;
	}

	public TextField getTfItem()
	{
		return tfItem;
	}

	public void setTfItem(TextField tfItem)
	{
		this.tfItem=tfItem;
	}


	
}
