package jeeafondo.webform;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Arrays;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jeeafondo.webform.annotations.OnEvent;

@WebServlet("*.wf")
public class WebFormServlet extends HttpServlet
{	
	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException,IOException
	{	
		try
		{
			// obtengo el callback method que va a procesar a este evento y su argumento
			String[] cm = _obtenerCallbackMethod(request);
			
			// obtengo el nombre del formulario en la sesion
			String sn = _obtenerSessionFormName(request); 
			
			// obtengo el form y le asigno los valores que llegan por request
			WebForm form = (WebForm)request.getSession().getAttribute(sn);
			form.setParamValues(request);
			
			// reseteo los errores
			form.clearErrors();
			
			// verifico que hay que ejecutar las validaciones
			boolean validate = _aplicanValidaciones(sn,cm[0],request);
			
			if( !validate || (form.validateControls() && form.validate()) )
			{				
				if( cm[1]==null )
				{
					Method  mtd = form.getClass().getMethod(cm[0]);
					mtd.invoke(form);
				}
				else
				{
					Method  mtd = form.getClass().getMethod(cm[0],String.class);
					mtd.invoke(form,cm[1]);					
				}
			}
			
			// regreso a la pagina que contiene al form
			request.getRequestDispatcher(form.getPage()).forward(request,response);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			throw new RuntimeException(ex);
		}	
	}

	private boolean _aplicanValidaciones(String nameInSession,String mtdName, HttpServletRequest request)
	{        
		String enableValidationsParamName = nameInSession+".enableValidations";
		String sEnabled[] = request.getParameterValues(enableValidationsParamName);
		boolean bEnabled = sEnabled==null?false:Arrays.asList(sEnabled).contains(mtdName+".wf");
		
		return bEnabled;
	}

	private String _obtenerSessionFormName(HttpServletRequest request)
	{
		// el nombre del objeto en la session lo obtengo asi:
		String x = request.getParameterNames().nextElement();
		String nameInSession = x.substring(0,x.lastIndexOf('.'));
		return nameInSession;
	}

	private String[] _obtenerCallbackMethod(HttpServletRequest request)
	{
		// URI = contextPath/recurso?argumentos
		String uri = request.getRequestURI();
		String mtdName = uri.substring(uri.lastIndexOf('/')+1,uri.lastIndexOf('.'));
		int posArg=mtdName.indexOf('-');
		
		String[] ret=new String[2];
		if( posArg<0 )
		{
			ret[0]=mtdName;
			ret[1]=null;
		}
		else
		{
			ret[0]=mtdName.substring(0,posArg);
			ret[1]=mtdName.substring(posArg+1);
		}

		return ret;
	}
}
